#pragma once

/// Object able to change values in function of a time-stamp.
class Animator
{
  public:
    /// @name Initialization
    //@{
    /// Default constructor
    Animator();
    //@}

    /// @name Access
    //@{
    /// Value of the current time-stamp in seconds.
    float time() const;
    /// Rate of change of the angle over time, expressed in degree per second
    float angleVelocityDegreePerSecond() const;

    /// Value of angle in degree for current time-stamp
    float currentAngleDegree() const;
    //@}

    /// @name Element change
    //@{
    /** \brief Replace currentAngleDegree() with `a'
     * - \post is_set: currentAngleDegree()==a
     */
    void setCurrentAngleDegree(float a);

    /** \brief Replace angleVelocityDegreePerSecond() with `velocity'
     * - \post is_set: angleVelocityDegreePerSecond()==velocity
     */
    void setAngleVelocityDegreePerSecond(float velocity);
    //@}

    /// @name Basic operations
    //@{
    /** \brief Replace time() with `t'.
     * - \post is_set: time()==t
     */
    void setTime(float t);
    /** \brief Update the animated parameters to the next time stamp
     * - \post changed: time()==new_time
     */
    void animate(float new_time);
    //@}

  protected:
    float m_time = 0.0f;
    float m_currentAngleDegree = 0.0f;
    float m_angleVelocityDegreePerSecond = 1.0f;
};
