#include "MyQGLWidget.h"

#include "Renderer.h"
#include "Animator.h"
#include <QElapsedTimer>
#include <QTimer>
#include <cassert>
#include <iostream>

MyQGLWidget::MyQGLWidget() : QGLWidget()
{
    m_fpsTimer = new QTimer;
    connect(m_fpsTimer, &QTimer::timeout, this, &MyQGLWidget::checkFPS);
    m_fpsTimer->start(m_fpsTimeInMs);

    m_elapsedTimer = new QElapsedTimer;
    m_tickTimer = new QTimer;
#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
    connect(m_tickTimer, &QTimer::timeout, this, &MyQGLWidget::animate);
#else
    connect(m_tickTimer, SIGNAL(timeout()), this, SLOT(animate()));
#endif
    m_tickTimer->start(1000.0f / 60.f); // 60Hz
}

MyQGLWidget::~MyQGLWidget()
{
    delete m_fpsTimer;
    delete m_tickTimer;
    delete m_elapsedTimer;
}

Renderer* MyQGLWidget::renderer() const
{
    return m_renderer;
}

void MyQGLWidget::setRenderer(Renderer* r)
{
    m_renderer = r;
    assert("post: is_set" && renderer() == r);
}

Animator* MyQGLWidget::animator() const
{
    return m_animator;
}

void MyQGLWidget::setAnimator(Animator* a)
{
    m_animator = a;
    if (m_animator != nullptr) {
        m_elapsedTimer->start();
        m_animator->setTime(milliSecondsToSeconds(m_elapsedTimer->elapsed()));
    }
    assert("post: is_set" && animator() == a);
}

void MyQGLWidget::initializeGL()
{
    QGLFormat f = format();
    std::cout << "format: stereo=" << f.stereo() << " alpha=" << f.alpha()
              << " sampleBuffer=" << f.sampleBuffers()
              << " samples=" << f.samples() << " accum=" << f.accum()
              << " depth=" << f.depth() << " stencil=" << f.stencil()
              << std::endl;

    if (m_renderer != nullptr) {
        m_renderer->initializeGL();
    }
}

void MyQGLWidget::paintGL()
{
    if (m_renderer != nullptr) {
        m_renderer->paintGL();
    }
}

void MyQGLWidget::resizeGL(int w, int h)
{
    if (m_renderer != nullptr) {
        m_renderer->resizeGL(w, h);
    }
}

void MyQGLWidget::checkFPS()
{
    if (m_renderer != nullptr) {
        int counter = m_renderer->paintGLCounter();
        m_renderer->resetPaintGLCounter();
        std::cout << "FPS=" << (counter / (m_fpsTimeInMs / 1000.0f))
                  << " frametime=" << (m_fpsTimeInMs / counter) << " ms"
                  << std::endl;
    }
}

void MyQGLWidget::animate()
{
    if (m_animator != nullptr) {
        m_animator->animate(milliSecondsToSeconds(m_elapsedTimer->elapsed()));
        updateGL();
    }
}

float MyQGLWidget::milliSecondsToSeconds(float milli) const
{
    return milli / 1000.0f;
}

#include "MyQGLWidget.moc"
