#include "MyQOpenGLWidget.h"

#include "Renderer.h"
#include "Animator.h"
#include <QElapsedTimer>
#include <QTimer>
#include <cassert>
#include <iostream>
#include "qtutil.h"

MyQOpenGLWidget::MyQOpenGLWidget() : QOpenGLWidget()
{
    m_fpsTimer = new QTimer;
    connect(m_fpsTimer, &QTimer::timeout, this, &MyQOpenGLWidget::checkFPS);
    m_fpsTimer->start(m_fpsTimeInMs);

    m_elapsedTimer = new QElapsedTimer;
    m_tickTimer = new QTimer;
    connect(m_tickTimer, &QTimer::timeout, this, &MyQOpenGLWidget::animate);
    m_tickTimer->start(1000.0f / 60.f); // 60Hz
}

MyQOpenGLWidget::~MyQOpenGLWidget()
{
    delete m_fpsTimer;
    delete m_tickTimer;
    delete m_elapsedTimer;
}

Renderer* MyQOpenGLWidget::renderer() const
{
    return m_renderer;
}

void MyQOpenGLWidget::setRenderer(Renderer* r)
{
    m_renderer = r;
    assert("post: is_set" && renderer() == r);
}

Animator* MyQOpenGLWidget::animator() const
{
    return m_animator;
}

void MyQOpenGLWidget::setAnimator(Animator* a)
{
    m_animator = a;
    if (m_animator != nullptr) {
        m_elapsedTimer->start();
        m_animator->setTime(milliSecondsToSeconds(m_elapsedTimer->elapsed()));
    }
    assert("post: is_set" && animator() == a);
}

void MyQOpenGLWidget::initializeGL()
{
    QSurfaceFormat f = format();
    std::cout << "format: stereo=" << f.stereo()
              << " alphaBufferSize=" << f.alphaBufferSize()
              << " samples=" << f.samples()
              << " depthBufferSize=" << f.depthBufferSize()
              << " stencilBufferSize=" << f.stencilBufferSize() << std::endl;

    if (m_renderer != nullptr) {
        m_renderer->initializeGL();
    }
}

void MyQOpenGLWidget::paintGL()
{
    if (m_renderer != nullptr) {
        m_renderer->paintGL();
    }
}

void MyQOpenGLWidget::resizeGL(int w_points, int h_points)
{
    if (m_renderer != nullptr) {
        double ratio = widgetDevicePixelRatio(this);
        m_renderer->resizeGL(static_cast<int>(w_points * ratio),
                             static_cast<int>(h_points * ratio));
    }
}

void MyQOpenGLWidget::checkFPS()
{
    if (m_renderer != nullptr) {
        int counter = m_renderer->paintGLCounter();
        m_renderer->resetPaintGLCounter();
        std::cout << "FPS=" << (counter / (m_fpsTimeInMs / 1000.0f))
                  << " frametime=" << (m_fpsTimeInMs / counter) << " ms"
                  << std::endl;
    }
}

void MyQOpenGLWidget::animate()
{
    if (m_animator != nullptr) {
        m_animator->animate(milliSecondsToSeconds(m_elapsedTimer->elapsed()));
        update();
    }
}

float MyQOpenGLWidget::milliSecondsToSeconds(float milli) const
{
    return milli / 1000.0f;
}

