#pragma once
#include <QOpenGLWidget>

class Renderer;
class Animator;
class QTimer;
class QElapsedTimer;

class MyQOpenGLWidget : public QOpenGLWidget
{
    Q_OBJECT
  public:
    /// @name Initialization
    //@{
    /** \brief Default constructor.
     * - \post no_renderer: renderer()==nullptr
     */
    MyQOpenGLWidget();
    //@}

    /// @name Removal
    //@{
    /// Destructor.
    ~MyQOpenGLWidget() override;
    //@}

    /// @name Access
    //@{
    /// Object performing the rendering.
    Renderer* renderer() const;

    /// Object that computes time dependent parameters.
    Animator* animator() const;
    //@}

    /// @name Element change
    //@{
    /** \brief Replace renderer() with `r'.
     * - \post is_set: renderer()==r
     */
    void setRenderer(Renderer* r);

    /** \brief Replace animator() with `a'
     * - \post is_set: animator()==a
     */
    void setAnimator(Animator* a);
    //@}

  protected:
    /// @name Implementation
    //@{
    /// Overridden from superclass to delegate to renderer() if any.
    void initializeGL() override;

    /// Overridden from superclass to delegate to renderer() if any.
    void paintGL() override;

    /// Overridden from superclass to delegate to renderer() if any.
    void resizeGL(int w_points, int h_points) override;

    /// Convert milliseconds to seconds.
    float milliSecondsToSeconds(float milli) const;
    //@}

    Renderer* m_renderer = nullptr;
    Animator* m_animator = nullptr;
    QTimer* m_tickTimer = nullptr;
    QElapsedTimer* m_elapsedTimer = nullptr;
    QTimer* m_fpsTimer = nullptr;
    double m_fpsTimeInMs = 5000.0; // 5s

  protected slots:
    /// @name Slots
    //@{
    /// Print average FPS for the last 5 seconds.
    void checkFPS();
    /// Perform an animation step. Triggered by m_timerTick timeout signal.
    void animate();
    //@}
};
