#pragma once

class RendererPrivate; // to avoid GLEW and QOpenGL conflicts
class Animator;

class Renderer
{
  public:
    /// @name Initialization
    //@{
    /** \brief Default constructor. Build the private object.
     * - \post counter_is_zero: paintGLCounter()==0
     */
    Renderer();
    //@}

    /// @name Removal
    //@{
    /// Destructor.
    ~Renderer();
    //@}

    /// @name Access
    //@{
    /// Object that computes time dependent parameters.
    Animator* animator() const;
    //@}

    /// @name Measurement
    //@{
    /** \brief Number of calls to paintGL() since the last call to
     * resetPaintGLCounter()
     */
    int paintGLCounter() const;
    //@}

    /// @name Element change
    //@{
    /** \brief Replace animator() with `a'
     * - \post is_set: animator()==a
     */
    void setAnimator(Animator* a);

    /** \brief Reset paintGLCounter()
     * - \post is_zero: paintGLCounter()==0
     */
    void resetPaintGLCounter();
    //@}

    /// @name Basic operations
    //@{
    /// Same semantic as QGLWidget::initializeGL()
    void initializeGL();
    /** Same semantic as QGLWidget::paintGL()
     * Also increment paintGLCounter().
     */
    void paintGL();
    /// Same semantic as QGLWidget::resizeGL()
    void resizeGL(int w, int h);
    //@}

  protected:
    RendererPrivate* m_private = nullptr;
    int m_paintGLCounter = 0;
};
