#include "RendererPrivate.h"

#include "Animator.h"

#include <OpenGL/gl.h> // #include <GL/glew.h>
#include <cassert>
#include <iostream>

RendererPrivate::RendererPrivate()
{
}

Animator* RendererPrivate::animator() const
{
    return m_animator;
}

void RendererPrivate::setAnimator(Animator* a)
{
    m_animator = a;
    assert("post: is_set" && animator() == a);
}

void RendererPrivate::initializeGL()
{
  //    glewInit();
    const GLubyte* renderer = glGetString(GL_RENDERER);
    const GLubyte* vendor = glGetString(GL_VENDOR);
    const GLubyte* version = glGetString(GL_VERSION);
    std::cerr << "renderer=" << renderer << "\nvendor=" << vendor
              << "\nversion=" << version << std::endl;
}

void RendererPrivate::paintGL()
{
    if (m_width > 0 && m_height > 0) {
        glViewport(0, 0, m_width, m_height);
        glClearColor(0.0f, 0.2f, 0.0f, 0.0f);
        glClearDepth(1.0f);
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

        glMatrixMode(GL_PROJECTION);
        glPushMatrix();
        glLoadIdentity();

        glMatrixMode(GL_MODELVIEW);
        glPushMatrix();
        glLoadIdentity();
        if (m_animator != nullptr) {
            glRotatef(m_animator->currentAngleDegree(), 0.0f, 0.0f, 1.0f);
        }
        glColor3f(1.0f, 0.0f, 0.0f);
        glBegin(GL_TRIANGLES);
        glVertex2f(0.0f, 0.0f);
        glVertex2f(1.0f, 0.0f);
        glVertex2f(1.0f, 1.0f);
        glEnd();
        glPopMatrix();

        glMatrixMode(GL_PROJECTION);
        glPopMatrix();
    }
}

void RendererPrivate::resizeGL(int w, int h)
{
    m_width = w;
    m_height = h;
}
