#include "qtutil.h"

#include <cassert>
#include <QWidget>
#include <QWindow>
#include <QApplication>
#include <QScreen>

QWindow* nativeWindowHandle(const QWidget* w)
{
    assert("pre: w_exists" && w != nullptr);
    QWindow* result = w->windowHandle();
    if (result == nullptr) {
        QWidget* p = w->nativeParentWidget();
        if (p != nullptr) {
            result = p->windowHandle();
        }
    }
    return result;
}

double widgetDevicePixelRatio(const QWidget* w)
{
    assert("pre: w_exists" && w != nullptr);
    double result = 1.0;
    QWindow* window = nativeWindowHandle(w);
    if (window != nullptr) {
        result = window->devicePixelRatio();
    } else {
        QCoreApplication* capp = QApplication::instance();
        QGuiApplication* app = dynamic_cast<QGuiApplication*>(capp);
        if (app != nullptr) {
            QScreen* primaryScreen = app->primaryScreen();
            if (primaryScreen != nullptr) {
                result = primaryScreen->devicePixelRatio();
            }
        }
    }
    return result;
}
